
/*******************************************************************************
*	DISCLAIMER
*
*	THIS SOFTWARE IS PROVIDED "AS IS" AND BFG ENGINEERING MAKES NO WARRANTIES
*	REGARDING THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING
*	BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
*	PURPOSE AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
*
*	WE (BFG ENGINEERING) DO NOT WARRANT THAT THIS SOFTWARE IS FREE FROM CLAIMS
*	BY A THIRD PARTY OF COPYRIGHT, PATENT, TRADEMARK, TRADE SECRET OR ANY OTHER
*	INTELLECTUAL PROPERTY INFRINGIMENT.
*
*	TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, BFG ENGINEERING
*	SHALL NOT BE LIABLE FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR
*	CONSEQUENTIAL DAMAGES FOR ANY REASON RELATED TO THIS SOFTWARE, EVEN IF BFG
*	ENGINEERING HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
*	PARTICULARLY, UNDER NO CIRCUMSTANCES ARE WE LIABLE FOR ANY OF THE FOLLOWING:
*	- THIRD PARTY CLAIMS AGAINST YOU FOR LOSSES OR DAMAGES;
*	- LOSS OF, OR DAMAGE TO YOUR RECORDS OR DATA; OR
*	- ECONOMIC CONSEQUENTIAL DAMAGES (INCLUDING LOST PROFITS OR SAVINGS) OR
*	  INCIDENTAL DAMAGES, EVEN IF WE ARE INFORMED OF THEIR POSSIBILITY.
*
*	BFG ENGINEERING DOES NOT WARRANT UNINTERRUPTED OR ERROR_FREE OPERATION OF
*	THIS SOFTWARE.
*	BFG ENGINEERING HAS NO OBLIGATION TO PROVIDE SERVICE, DEFECT CORRECTION OR
*	ANY MANTEINANCE OF THIS SOFTWARE; OR TO SUPPLY ANY UPDATES OR ENHANCEMENTS
*	EVEN IF SUCH ARE, OR LATER BECOME, AVAILABLE.
*
*	IF YOU DOWNLOAD OR USE THIS SOFTWARE YOU AGREE TO THESE TERMS, AND DO IT AT
*	YOUR OWN RISK.
*******************************************************************************/

/*******************************************************************************
*	File:		mcrplib.h
*	Project:	MCRP09_RX200
*	Version:	1.0
*	Date:		19-11-2013 (DD-MM-YYYY)
*	Author:		Andrea Ghirardello
*	Contents:	fixed point motor control library
*******************************************************************************/

/* THIS FILE IS INCLUDED ALSO IN LIBRARY SO NOTHING CAN BE MODIFIED! */

#ifndef _MCRPLIB_H
#define _MCRPLIB_H

/******************************************************************************
*	Macro definitions
******************************************************************************/

// MISRA types inclusion
#ifndef _MISRA_H_INCLUDED_
#define _MISRA_H_INCLUDED_

typedef          char		char_t;
typedef unsigned char		uint8_t;
typedef unsigned short int  uint16_t;
typedef unsigned long int   uint32_t;
typedef signed   char		int8_t;
typedef signed   short int	int16_t;
typedef signed   long int	int32_t;
typedef          float		float32_t;
typedef          double		float64_t;

#ifdef TRUE
#undef TRUE
#endif

#ifdef FALSE
#undef FALSE
#endif

typedef enum bool {
	FALSE = 0,
	TRUE = 1
} bool_t;

#endif // _MISRA_H_INCLUDED_

// two pi
#ifndef TWOPIF
#define TWOPIF			( 6.283185307 )
#endif	// TWOPIF

// base value for normalization
#ifndef NORMVAL
#define SH_NORMVAL		( 14 )			// NORMVAL=2^SHFT_NORMVAL
#define NORMVAL			((int16_t)( 1 << SH_NORMVAL ))
#define NORMVALF		((float32_t)NORMVAL)
#endif	// NORMVAL

// numerical constants
#ifndef S16_CONST
#define S16_CONST
#define	SH_S16_CONST	( SH_NORMVAL )	// amplification shifts
#define	AMP_S16_CONST	( 1 << SH_S16_CONST )	// amplification
#define	ONETHIRD		( 5461 )		// 2^14*(1/3)=5461.33333
#define	TWOTHIRDS		( 10923 )		// 2^14*(2/3)=10922.66666
#define	SQRT3			( 28378 )		// 2^14*SQUAREROOT(3)=28377.92043
#define	TWODIVSQRT3		( 18919 )		// 2^15/SQUAREROOT(3)=18918.61362
#define	ONEDIVSQRT3		( 9459 )		// 2^14/SQUAREROOT(3)=9459.30681
#define	SQRT2			( 23170 )		// 2^14*SQUAREROOT(2)=23170.47501
#define	ONEDIVSQRT2		( 11585 )		// 2^14/SQUAREROOT(2)=11585.2375
#define SQRT3DIVSQRT2	( 20066 )		// 2^14*SQUAREROOT(3/2)=20066.21997
#define SQRT2DIVSQRT3	( 13377 )		// 2^14*SQUAREROOT(2/3)=13377.47998
#endif	// S16_CONST

// trigonometric function related definitions
#ifndef ANGLES
#define ANGLES

// normalizations
#define SH_NORMSIN		SH_NORMVAL
#define NORMSIN			((int16_t)( 1 << SH_NORMSIN ))
#define SH_NORMTAN		SH_NORMVAL
#define NORMTAN			((int16_t)( 1 << SH_NORMTAN ))

// angles definitions
#define	PISIXTHS		( 5461 )	// 0x1555 030
#define	PIFOURTHS		( 8192 )	// 0x2000 045
#define	PITHIRDS		( 10923 )	// 0x2AAB 060
#define	PIHALVES		( 16384 )	// 0x4000 090
#define	TWOPITHIRDS		( 21845 )	// 0x5555 120
#define	THREEPIFOURTHS	( 24576 )	// 0x6000 135
#define	FIVEPISIXTHS	( 27307 )	// 0x6AAB 150
#define	PI				( 32768 )	// 0x8000 180
#define	SEVENPISIXTHS	( 38229 )	// 0x9555 210
#define	FIVEPIFOURTHS	( 40960 )	// 0xA000 225
#define	FOURPITHIRDS	( 43691 )	// 0xAAAB 240
#define	THREEPIHALVES	( 49152 )	// 0xC000 270
#define	FIVEPITHIRDS	( 54613 )	// 0xD555 300
#define	SEVENPIFOURTHS	( 57344 )	// 0xE000 315
#define	ELEVENPISIXTHS	( 60075 )	// 0xEAAB 330
#define	TWOPI			( 65536 )	// 0x00010000 LONG; IF WORD ACCESS GIVES 0
#define	ANGMOD			( 0xFFFF )	// TWOPI-1
#define	ANG1Q			( 0x3FFF )	// select first_quarter_value
#define	QUAMOD			( 0xC000 )	// select quarter
#define	ISCOS			( 0x4000 )	// table(first_quarter_value) gives cos
#define	ISNEG			( 0x8000 )	// sin is neg
#define KAF				(TWOPI / TWOPIF)  // angles conversion constant
#define KA				((uint16_t)KAF)	  // angles conversion constant

#endif	// ANGLES


/******************************************************************************
*	Functions prototypes
******************************************************************************/

/* TRIGONOMETRIC FUNCTIONS ***************************************************/

/******************************************************************************
* Name:			McrpLib_sin
* Description:	y=NORMSIN*sin(ang)
* Input:		ang=KA*angle[rad], 0<=ang<TWOPI (0<=angle<2pi)
* Output:		normalized sin value y, |y|<=NORMSIN
* Revision:		1.0
******************************************************************************/
int16_t McrpLib_sin(int16_t ang);

/******************************************************************************
* Name:			McrpLib_cos
* Description:	y=NORMSIN*cos(ang)
* Input:		ang=KA*angle[rad], 0<=ang<TWOPI (0<=angle<2pi)
* Output:		normalized cos value y, |y|<=NORMSIN
* Revision:		1.0
******************************************************************************/
int16_t McrpLib_cos(int16_t ang);

/******************************************************************************
* Name:			McrpLib_sinarcos
* Description:	y=NORMSIN*sin(arcos(x/NORMSIN))
* Input:		normalized cos(angle) value x, 0<=x<=NORMSIN, (0<=angle<=pi/2)
* Output:		normalized sin(angle) value y, 0<=y<=NORMSIN
* Revision:		1.0
******************************************************************************/
int16_t McrpLib_sinarcos(int16_t x);

/******************************************************************************
* Name:			McrpLib_atan2
* Description:	ang=KA*arctan(y/x)
* Input:		amplified value x=A*cos(angle),
*				amplified value y=A*sin(angle),
*				0<A<2^15, (0<=angle<=2pi)
* Output:		internal representation value of angle: ang=KA*angle
* Revision:		1.0
******************************************************************************/
int16_t McrpLib_atan2(int16_t x, int16_t y);

/* END OF TRIGONOMETRIC FUNCTIONS ********************************************/


/* TRANSFORMATION FUNCTIONS **************************************************/

/******************************************************************************
* Name:			McrpLib_AngleSet
* Description:	internal angle variable setting
* Input:		ang=KA*angle[rad], 0<=ang<TWOPI (0<=angle<2pi)
* Output:		nothing
* Modifies:		internal_angle (base for Park transformations)
* Revision:		1.0
******************************************************************************/
void McrpLib_AngleSet(int16_t ang);

/******************************************************************************
* Name:			McrpLib_uv_alphabeta
* Description:	unitary gain transformation (u, v, (w=-u-v))->(alpha, beta):
*				alpha=u, beta=(u+2*v)/sqrt(3)
* Input:		u and v components,
*				alpha component address, beta component address
* Output:		nothing
* Modifies:		alpha, beta components
* Note:			no omopolar component considered (u+v+w=0 supposed)!
* Revision:		1.0
******************************************************************************/
void McrpLib_uv_alphabeta(	int16_t u, int16_t v,
				   			int16_t *alpha, int16_t *beta);

/******************************************************************************
* Name:			McrpLib_uvw_alphabeta
* Description:	unitary gain transformation (u, v, w)->(alpha, beta):
*				alpha=(2u-v-w)/3, beta=(v-w)/sqrt(3)
* Input:		u, v, w components,
*				alpha component address, beta component address
* Output:		nothing
* Modifies:		alpha, beta components
* Note:			no limitation (omopolar component kept into account)
* Revision:		1.0
******************************************************************************/
void McrpLib_uvw_alphabeta(	int16_t u, int16_t v, int16_t w,
				   			int16_t *alpha, int16_t *beta);

/******************************************************************************
* Name:			McrpLib_alphabeta_uv
* Description:	unitary gain transformation (alpha, beta)->(u, v, (w=-u-v)):
*				u=alpha, v=((sqrt(3)*beta-alpha)/2
* Input:		alpha and beta components,
*				u component address, v component address
* Output:		nothing
* Modifies:		u, v components
* Revision:		1.0
******************************************************************************/
void McrpLib_alphabeta_uv(	int16_t alpha,	int16_t beta,
							int16_t *u,		int16_t *v);

/******************************************************************************
* Name:			McrpLib_alphabeta_dq
* Description:	unitary gain transformation (alpha, beta)->(d, q):
*				d=alpha*cos(internal_angle)+beta*sin(internal_angle)
*				q=alpha*cos(internal_angle)-beta*sin(internal_angle)
* Input:		alpha component, beta component,
*				d component address, q component address
* Output:		nothing
* Modifies:		d, q components
* Note:			requires previously set internal variables sinang, cosang
* Revision:		1.0
******************************************************************************/
void McrpLib_alphabeta_dq(	int16_t alpha,	int16_t beta,
							int16_t *d,		int16_t *q);

/******************************************************************************
* Name:			McrpLib_dq_alphabeta
* Description:	unitary gain transformation (d, q)->(alpha, beta):
*				alpha=d*cos(internal_angle)-q*sin(internal_angle)
*				beta=d*sin(internal_angle)+q*cos(internal_angle)
* Input:		d component, q component,
*				alpha component address, beta component address
* Output:		nothing
* Modifies:		alpha, beta components
* Note:			requires previously set internal variables sinang, cosang
* Revision:		1.0
******************************************************************************/
void McrpLib_dq_alphabeta(	int16_t d, 		int16_t q,
							int16_t *alpha,	int16_t *beta);

/******************************************************************************
* Name:			McrpLib_xy_rt
* Description:	transformation (x, y)->(ro, theta)
* Input:		x component, y component,
*				ro component address, theta component address
* Output:		nothing
* Modifies:		ro (amplitude), theta (argument) components
* Revision:		1.0
******************************************************************************/
void McrpLib_xy_rt(	int16_t x,	int16_t y,
					int16_t *r,	int16_t *t);

/******************************************************************************
* Name:			McrpLib_rt_xy
* Description:	transformation (ro, theta)->(x, y)
* Input:		ro (amplitude) component, theta (argument) component,
*				x component address, y component address
* Output:		nothing
* Modifies:		x, y components
* Revision:		1.0
******************************************************************************/
void McrpLib_rt_xy(	int16_t r,	int16_t t,
					int16_t *x,	int16_t *y);
/* END OF TRANSFORMATION FUNCTIONS *******************************************/


/* FLUX AND SPEED ESTIMATION FUNCTIONS ***************************************/

/* IMPORTANT NOTE *************************************************************
* The calculation inside the flux estimation functions are based on the
* already described internal representation of physical quantities, and all the
* quantities are intended to be represented as described in "units.h".
* End of IMPORTANT NOTE ******************************************************/

/******************************************************************************
* Name:			McrpLib_SetBaseValues
* Description:	Set base values, in order to allow the library to calculate
*				some internal constants
* Input:		sampling frequency fs[Hz] (it is the time conversion constant KT)
*				base voltage bv[Vol]
*				base current bc[Amp]
*				base speed bs[rad/sec]
* Output:		nothing
* Modifies:		internal parameters
* Revision:		1.0
******************************************************************************/
void McrpLib_SetBaseValues(float32_t fs, float32_t bv, float32_t bc, float32_t bs);

/******************************************************************************
* Name:			McrpLib_FESetPar
* Description:	flux estimation parameters setting
* Input:		stator resistance r [int]
*				synchronous inductance l [int]
*				permanent magnets flux amplitude f [int]
*				flux feedback gain g
* Output:		nothing
* Modifies:		internal parameters
* Revision:		1.0
******************************************************************************/
void McrpLib_FESetPar(int16_t r, int16_t l, int16_t f, int16_t g);

/******************************************************************************
* Name:			McrpLib_FluxEst
* Description:	permanent magnets flux vector estimation
* Input:		alpha voltage component va, beta voltage component vb,
*				alpha current component ia, beta current component ib,
*				permanent magnets flux vector amplitude address *m
*				permanent magnets flux vector phase address *p
* Output:		nothing
* Modifies:		permanent magnets flux vector amplitude m
*				permanent magnets flux vector argument (phase) p
* Revision:		1.0
******************************************************************************/
void McrpLib_FluxEst(	int16_t va, int16_t vb,
						int16_t ia, int16_t ib,
						int16_t *m, int16_t *p);

/******************************************************************************
* Name:			McrpLib_FluxSet
* Description:	permanent magnets flux vector estimation algorithm pre-set
* Input:		alpha voltage component va, beta voltage component vb,
*				alpha current component ia, beta current component ib,
*				permanent magnets flux vector amplitude mo, and phase ph
* Output:		nothing
* Modifies:		internal memories
* Revision:		1.0
******************************************************************************/
void McrpLib_FluxSet(	int16_t va, int16_t vb,
						int16_t ia, int16_t ib,
						int16_t mo, int16_t ph);

/******************************************************************************
* Name:			McrpLib_FEASetPar
* Description:	flux estimation (with approx. integral) parameters setting
* Input:		stator resistance r [int],
*				synchronous inductance l [int],
*				approximation filter time constant t [int]
* Output:		nothing
* Modifies:		internal parameters
* Revision:		1.0
******************************************************************************/
void McrpLib_FEASetPar(	int16_t r, int16_t l, int16_t t);

/******************************************************************************
* Name:			McrpLib_FluxEstApp
* Description:	magnetic flux vector estimation with approximated integral
* Input:		alpha voltage component va [int],
*				beta voltage component vb [int],
*				alpha current component ia [int],
*				beta current component ib [int],
*				permanent magnets flux vector amplitude famp [int] address,
*				permanent magnets flux vector argument (phase) farg [int] address
* Output:		nothing
* Modifies:		flux amplitude and argument
*				internal filters memories
* Revision:		1.0
******************************************************************************/
void McrpLib_FluxEstApp(int16_t va, int16_t vb,
						int16_t ia, int16_t ib,
						int16_t *famp, int16_t *farg);

/******************************************************************************
* Name:			McrpLib_FluxSetApp
* Description:	permanent magnets flux vector estimation algorithm pre-set
* Input:		alpha voltage component va [int],
*				beta voltage component vb [int],
*				alpha current component ia [int],
*				beta current component ib [int],
*				permanent magnets flux vector amplitude famp [int],
*				permanent magnets flux vector argument (phase) farg [int]
* Output:		nothing
* Modifies:		internal memories
* Revision:		1.0
******************************************************************************/
void McrpLib_FluxSetApp(int16_t va, int16_t vb,
						int16_t ia, int16_t ib,
						int16_t famp, int16_t farg);

/*******************************************************************************
* Name:			McrpLib_SpeedEst
* Description:	speed estimation: speed is expressed as difference between
*				two phase samples, amplified by NORMVAL
* Input:		actual phase [int]
* Output:		estimated speed [int]
* Revision:		1.0
*******************************************************************************/
int32_t McrpLib_SpeedEst(int16_t phase);

/******************************************************************************
* Name:			McrpLib_SpeedSet
* Description:	speed estimation algorithm pre-set
* Input:		speed [int], phase[int]
* Output:		nothing
* Modifies:		internal memories
* Revision:		1.0
******************************************************************************/
void McrpLib_SpeedSet(int32_t speed, int16_t phase);

/* END OF FLUX AND SPEED ESTIMATION FUNCTIONS ********************************/


/* SINGLE SHUNT MANAGEMENT FUNCTIONS *****************************************/

// state values for single shunt current detection
#ifndef SS_STATES
#define SS_STATES
#define CSS_NOTHING			( 0 )
#define CSS_PIUPIV			( 112 )
#define CSS_PIUPIW			( 113 )
#define CSS_PIVPIU			( 121 )
#define CSS_PIVPIW			( 123 )
#define CSS_PIWPIU			( 131 )
#define CSS_PIWPIV			( 132 )
#define CSS_MIUPIV			( 212 )
#define CSS_MIUPIW			( 213 )
#define CSS_MIVPIU			( 221 )
#define CSS_MIVPIW			( 223 )
#define CSS_MIWPIU			( 231 )
#define CSS_MIWPIV			( 232 )
#define CSS_MIUMIV			( 312 )
#define CSS_MIUMIW			( 313 )
#define CSS_MIVMIU			( 321 )
#define CSS_MIVMIW			( 323 )
#define CSS_MIWMIU			( 331 )
#define CSS_MIWMIV			( 332 )
#define CSS_PIUMIV			( 412 )
#define CSS_PIUMIW			( 413 )
#define CSS_PIVMIU			( 421 )
#define CSS_PIVMIW			( 423 )
#define CSS_PIWMIU			( 431 )
#define CSS_PIWMIV			( 432 )
#endif	// SS_STATES

/******************************************************************************
* Name:			McrpLib_SetPar_SS
* Description:	single shunt algorithm internal constants setting
* Input:		(all in internal time units -MTU ticks-)
*				delss required time between A/D command and next commutation
*				minss minimum time between two commutations
*				smper half pwm period
*				ddtim deadtimes
* Output:		nothing
* Modifies:		internal memories
* Revision:		1.0
******************************************************************************/
void McrpLib_SetPar_SS(	uint16_t delss, uint16_t minss,
						uint16_t smper, uint16_t ddtim);

/******************************************************************************
* Name:			McrpLib_CalDuties_SS
* Description:	single shunt duty cycles and A/D timing setting
* Input:		du, dv, dw required duty cycles
*				u1, v1, w1, u2, v2, w2 duties to impose addresses
*				c1, c2 A/D conversion times addresses
*				st current reading status (indicates which are the currents
*				which will be read)
*				flag to avoid deep compensation
* Output:		nothing
* Modifies:		u1, v1, w1, u2, v2, w2, c1, c2, st
* Revision:		1.0
******************************************************************************/
void McrpLib_CalDuties_SS(	int16_t du0, int16_t dv0, int16_t dw0,
							int16_t *u1, int16_t *v1, int16_t *w1,
							int16_t *u2, int16_t *v2, int16_t *w2,
							int16_t *c1, int16_t *c2,
							uint16_t *st, uint16_t nocomp);

/* END OF SINGLE SHUNT MANAGEMENT FUNCTIONS **********************************/

#endif /* _MCRPLIB_H */

