
/*******************************************************************************
*	DISCLAIMER
*
*	THIS SOFTWARE IS PROVIDED "AS IS" AND BFG ENGINEERING MAKES NO WARRANTIES
*	REGARDING THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING
*	BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
*	PURPOSE AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
*
*	WE (BFG ENGINEERING) DO NOT WARRANT THAT THIS SOFTWARE IS FREE FROM CLAIMS
*	BY A THIRD PARTY OF COPYRIGHT, PATENT, TRADEMARK, TRADE SECRET OR ANY OTHER
*	INTELLECTUAL PROPERTY INFRINGIMENT.
*
*	TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, BFG ENGINEERING
*	SHALL NOT BE LIABLE FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR
*	CONSEQUENTIAL DAMAGES FOR ANY REASON RELATED TO THIS SOFTWARE, EVEN IF BFG
*	ENGINEERING HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
*	PARTICULARLY, UNDER NO CIRCUMSTANCES ARE WE LIABLE FOR ANY OF THE FOLLOWING:
*	- THIRD PARTY CLAIMS AGAINST YOU FOR LOSSES OR DAMAGES;
*	- LOSS OF, OR DAMAGE TO YOUR RECORDS OR DATA; OR
*	- ECONOMIC CONSEQUENTIAL DAMAGES (INCLUDING LOST PROFITS OR SAVINGS) OR
*	  INCIDENTAL DAMAGES, EVEN IF WE ARE INFORMED OF THEIR POSSIBILITY.
*
*	BFG ENGINEERING DOES NOT WARRANT UNINTERRUPTED OR ERROR_FREE OPERATION OF
*	THIS SOFTWARE.
*	BFG ENGINEERING HAS NO OBLIGATION TO PROVIDE SERVICE, DEFECT CORRECTION OR
*	ANY MANTEINANCE OF THIS SOFTWARE; OR TO SUPPLY ANY UPDATES OR ENHANCEMENTS
*	EVEN IF SUCH ARE, OR LATER BECOME, AVAILABLE.
*
*	IF YOU DOWNLOAD OR USE THIS SOFTWARE YOU AGREE TO THESE TERMS, AND DO IT AT
*	YOUR OWN RISK.
*******************************************************************************/

/*******************************************************************************
*	File:		customize.h
*	Project:	YRotateItRX111_3s_V1
*	Version:	1.0
*	Date:		01-07-2014 (DD-MM-YYYY)
*	Author:		Andrea Ghirardello
*	Contents:	Custom values definition
*******************************************************************************/

#ifndef _CUSTOMIZE_H
#define _CUSTOMIZE_H

// internal (E6125A) or external (E6108A) power stage selection
#include "pws_E6125A.h"				// INTERNAL POWER STAGE DEFINITION FILE
//#include "pws_E6108A.h"			// EXTERNAL POWER STAGE DEFINITION FILE

// eeprom (comment if not used)
#define EEPROM_USED

// debug (comment after first control)
#define MACRO_DEBUG

// open loop debug (comment in final release)
// if not commented it can be enabled writing 55 in par. 0
#define DEBUG_OPENLOOP

// DC current vector setting debug mode (comment if not used)
// if not commented it can be enabled writing 66 in par. 0
#define DCCUR_SETTING

// ADDITIONAL FEATURE: current PI manual tuning (comment if not used)
#define CPI_TUN

// ADDITIONAL FEATURE: current PI automatic tuning (comment if not used)
#define CPI_AUT

// ADDITIONAL FEATURE: motor parameters auto-identification (comment if not used)
#define MOT_IDENT

// ADDITIONAL FEATURE: oscilloscope window in GUI management (comment if not used)
#define OSC_WIN

// lower arms ON when not rotating
//#define DEF_LOW_ON

// PI gains internal (de-)amplification
// Note:	real proportional gain is (parameter_value/G_PRO)
//			real integral gain is (parameter_value/G_INT)*(8000/sam_fre_Hz)
#define SH_INT				(  6 )
#define SH_PRO 				( 10 )
#define G_INT				(1 << SH_INT)
#define G_PRO				(1 << SH_PRO)

// parameters measurement units resolution
#define	SEC_RES				(  1000 )	// time parameters expressed in sec/SEC_RES
#define	AMP_RES				(  1000 )	// current parameters expressed in ampere/AMP_RES
#define	VOL_RES				(    10 )	// voltage parameters expressed in volt/VOL_DIV
#define	OHM_RES				(   100 )	// resistance parameters expressed in ohm/OHM_RES
#define	HEN_RES				( 10000 )	// inductance parameters expressed in henry/HEN_RES
#define	WEB_RES				( 10000 )	// flux parameters expressed in weber/WEB_RES

// motor parameters default values include
#include "defpar.h"

/* Nanotec DB42S03 24V 4000rpm motor default parameters
#define	RPM_MIN_DEF			(   500 )	//  200 <= X <=  5000	// min speed in rpm
#define	RPM_MAX_DEF			(  8000 )	// 1000 <= X <= 20000	// max speed in rpm
#define	R_ACC_DEF			(  2000 )	//    1 <= X <= 10000	// accel. ramp in rpm/sec
#define	R_DEC_DEF			(  2000 )	//    1 <= X <= 10000	// accel. ramp in rpm/sec
#define	C_POLI_DEF			(     4 )	//    1 <= X <=    24	// polar couples number
#define	I_START_DEF			(   400 )	//    0 <= X <= 15000	// startup current in A(pk)/AMP_RES
#define	I_MAX_DEF			(  2000 )	//    0 <= X <= 15000	// max current in A(pk)/AMP_RES
#define	R_STA_DEF			(    80 )	//    0 <= X <= 30000	// stator phase resistance in Ohm/OHM_RES
#define	L_SYN_DEF			(     6 )	//    0 <= X <= 30000	// synchronous inductance in Henry/HEN_RES
#define PM_FLX_DEF			(    64 )	//	  0 <= X <= 30000	// permanent magnet flux in Weber/WEB_RES
#define	KP_CUR_DEF			(   600 )	//    0 <= X <= 10000	// K prop. current control
#define	KI_CUR_DEF			(  4000 )	//    0 <= X <= 10000	// K integ. current control
#define	KP_VEL_DEF			(  1000 )	//    0 <= X <= 10000	// K prop. speed control
#define	KI_VEL_DEF			(   500 )	//    0 <= X <= 10000	// K integ. speed control
#define	FB_GAIN_DEF			(    10 )	//    0 <= X <= 30000	// flux estimation feedback gain (exact est.)
#define PHA_OFF_DEF			(     0 )	//	  0 <= X <=   360	// offset angle
#define	ST_TIM_DEF			(  1000 )	//    0 <= X <= 30000	// startup acceleration time in sec/SEC_RES
#define	FLX_FTAU_DEF		(   180 )	//    8 <= X <=  1000	// flux estimation filter time constant in sec/SEC_RES (approx. est.)
#define SAM_FRE_DEF			(  8000 )	// 4000 <= X <= 16000	// sampling frequency in Hz
#define F_RATIO_DEF			(     2 )	//    1 <= X <=     4	// pwm_freq/sam_freq ratio
*/

// startup timing definitions
#define STUP_CUR_RT_MS		(   200 )	// current rising time during startup alignment [ms]
#define STUP_CUR_FT_MS		(  1000 )	// d current falling time after startup [ms] (assuming id=i_start)

// rotation direction
//#define ROT_REV

// modulation
#define REFCOMP							// modulation compensation (if performed) is based on current references
#define CENTRED							// centred modulation
//#define OVERMOD						// over-modulation

// other control parameters
#define MAXERR_RPM			( 300 )		// maximum error for ramp limitation and PI clamping
#define K_MARGV0			((uint16_t)(0.90 * NORMVAL))	// max_d_volt/max_volt ratio

// speed PI error clamping
#define SPEEDPI_ERRCLAMP				// input to speed PI is clamped to MAXERR_RPM

// ramp clamps (comment if you don't want them)
#define RAMP_CL0						// clamp if heavy speed error

// flux weakening
#define FLUXWEAK						// comment to avoid flux weakening
#define K_MARGV1			((uint16_t)(0.95 * NORMVAL))	// fraction of available voltage at which FW begins
#define FW_CUR_RT_MS		( 2000 )	// ramp (0 -> ID_MAX) duration in ms

// flux estimation method choice
// exact integration (uses FB_GAIN) or approximated integration (uses FLX_FS)
// comment if you want approximated estimation
//#define EXACT

#endif	// _CUSTOMIZE_H

