
/*******************************************************************************
*	DISCLAIMER
*
*	THIS SOFTWARE IS PROVIDED "AS IS" AND BFG ENGINEERING MAKES NO WARRANTIES
*	REGARDING THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING
*	BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
*	PURPOSE AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
*
*	WE (BFG ENGINEERING) DO NOT WARRANT THAT THIS SOFTWARE IS FREE FROM CLAIMS
*	BY A THIRD PARTY OF COPYRIGHT, PATENT, TRADEMARK, TRADE SECRET OR ANY OTHER
*	INTELLECTUAL PROPERTY INFRINGIMENT.
*
*	TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, BFG ENGINEERING
*	SHALL NOT BE LIABLE FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR
*	CONSEQUENTIAL DAMAGES FOR ANY REASON RELATED TO THIS SOFTWARE, EVEN IF BFG
*	ENGINEERING HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
*	PARTICULARLY, UNDER NO CIRCUMSTANCES ARE WE LIABLE FOR ANY OF THE FOLLOWING:
*	- THIRD PARTY CLAIMS AGAINST YOU FOR LOSSES OR DAMAGES;
*	- LOSS OF, OR DAMAGE TO YOUR RECORDS OR DATA; OR
*	- ECONOMIC CONSEQUENTIAL DAMAGES (INCLUDING LOST PROFITS OR SAVINGS) OR
*	  INCIDENTAL DAMAGES, EVEN IF WE ARE INFORMED OF THEIR POSSIBILITY.
*
*	BFG ENGINEERING DOES NOT WARRANT UNINTERRUPTED OR ERROR_FREE OPERATION OF
*	THIS SOFTWARE.
*	BFG ENGINEERING HAS NO OBLIGATION TO PROVIDE SERVICE, DEFECT CORRECTION OR
*	ANY MANTEINANCE OF THIS SOFTWARE; OR TO SUPPLY ANY UPDATES OR ENHANCEMENTS
*	EVEN IF SUCH ARE, OR LATER BECOME, AVAILABLE.
*
*	IF YOU DOWNLOAD OR USE THIS SOFTWARE YOU AGREE TO THESE TERMS, AND DO IT AT
*	YOUR OWN RISK.
*******************************************************************************/

/*******************************************************************************
*	File:		globdef.h
*	Project:	YRotateItRX111_3s_V1
*	Version:	1.0
*	Date:		01-07-2014 (DD-MM-YYYY)
*	Author:		Andrea Ghirardello
*	Contents:	global definitions
*******************************************************************************/

#ifndef	_GLOBDEF_H
#define	_GLOBDEF_H

// inclusions
#include <machine.h>
#include <stdlib.h>
#include <mathf.h>
#include "typedefine.h"
#include "iodefine.h"
#include "globvar.h"
#include "mask.h"
#include "const_def.h"

/* TRUE/FALSE
#ifndef TRUE
	#define TRUE		( 1 )
#endif
#ifndef FALSE
	#define FALSE		( 0 )
#endif
*/

// NULL/NOP
#ifndef NULL
	#define NULL		((void *)0)
#endif                        
#ifndef NOP
	#define NOP			{ nop(); }
#endif                        

// Delay
#define US1				(    5 )
#define US10			(   50 )
#define US100			(  500 )
#define US1000			( 5000 )
#define DELAY(X, Y)		{	uint16_t x, y;					\
							for(x = X; x > 0; --x) {		\
								for(y = Y; y > 0; --y) {	\
									NOP				}	}	}
#define DELAY1US(X)		DELAY(X, US1)
#define DELAY10US(X)	DELAY(X, US10)
#define DELAY100US(X)	DELAY(X, US100)
#define DELAY1MS(X)		DELAY(X, US1000)

// 16bit mul_w(), mulu_w(), div_w(), divu_w() define
#ifndef MYMULDIV
#define MYMULDIV
#define	mul_w(X, Y)		(( int32_t)((( int16_t)( X )) * (( int16_t)( Y ))))
#define	mulu_w(X, Y)	((uint32_t)(((uint16_t)( X )) * ((uint16_t)( Y ))))
#define	div_w(X, Y)		(( int16_t)((( int32_t)( X )) / (( int16_t)( Y ))))
#define	divu_w(X, Y)	((uint16_t)(((uint32_t)( X )) / ((uint16_t)( Y ))))
#endif	// MYMULDIV

// macro for Module Stop Control
#define WAKEUP_S12AD	{	SYSTEM.PRCR.WORD = 0xA502;	/* enable writing PRC1 */	\
							MSTP(S12AD) = 0;			/* wake-up S12AD */			\
							SYSTEM.PRCR.WORD = 0xA500;	/* disable writing PRC1 */	}
#define WAKEUP_MTU		{	SYSTEM.PRCR.WORD = 0xA502;	/* enable writing PRC1 */	\
							MSTP(MTU) = 0;				/* wake-up MTU */			\
							SYSTEM.PRCR.WORD = 0xA500;	/* disable writing PRC1 */	}
#define WAKEUP_SCI1		{	SYSTEM.PRCR.WORD = 0xA502;	/* enable writing PRC1 */	\
							MSTP(SCI1) = 0;				/* wake-up SCI1 */			\
							SYSTEM.PRCR.WORD = 0xA500;	/* disable writing PRC1 */	}
#define WAKEUP_DTC		{	SYSTEM.PRCR.WORD = 0xA502;	/* enable writing PRC1 */	\
							MSTP(DTC) = 0;				/* wake-up DTC */			\
							SYSTEM.PRCR.WORD = 0xA500;	/* disable writing PRC1 */	}

// macro for A/D conversions
#define	S12AD_START		S12AD.ADCSR.BIT.ADST = 1;		// B7
#define	S12AD_STOP		S12AD.ADCSR.BIT.ADST = 0;		// B7
#define	S12AD_C_ON		(0 != S12AD.ADCSR.BIT.ADST)		// B7
#define	WAIT_S12AD		while(S12AD_C_ON);
#define AN0_IU			((uint16_t) S12AD.ADDR0)		// PIN45 P40 AN0	IN  IU
#define AN1_IV			((uint16_t) S12AD.ADDR1)		// PIN43 P41 AN1	IN  IV
#define AN2_IW			((uint16_t) S12AD.ADDR2)		// PIN42 P42 AN2	IN  IW
#define AN6_VB			((uint16_t) S12AD.ADDR6)		// PIN41 P46 AN6	IN  VB
#define AN9_TMP			((uint16_t) S12AD.ADDR9)		// PIN38 PE1 AN9	IN  TMP
#define AN10_AUX		((uint16_t) S12AD.ADDR10)		// PIN37 PE2 AN10	IN  AUX
#define SEL_IU_AD		S12AD.ADANSA.WORD = WSET0;
#define SEL_IV_AD		S12AD.ADANSA.WORD = WSET1;
#define SEL_IW_AD		S12AD.ADANSA.WORD = WSET2;
#define SEL_CUR_AD		S12AD.ADANSA.WORD = (WSET0 | WSET1 | WSET2);
#define SEL_VB_AD		S12AD.ADANSA.WORD = WSET6;
#define SEL_TMP_AD		S12AD.ADANSA.WORD = WSET9;
#define SEL_AUX_AD		S12AD.ADANSA.WORD = WSETA;

// macro for complementary pwm MTU timer 3, 4 (active state = high)
// PIN23 PC5 		MTIOC3B OUT PWM_UH
// PIN24 PC4 		MTIOC3D OUT PWM_UL
// PIN26 PB3 (PC2)	MTIOC4A	OUT PWM_VH
// PIN27 PB1 (PC1)	MTIOC4C	OUT PWM_VL
// PIN36 PE3 		MTIOC4B OUT PWM_WH
// PIN35 PE4 		MTIOC4D OUT PWM_WL
#define	BUF_SP			MTU.TCBRB	// buffer for TCDRB (semiper)
#define	BUF_SPD			MTU3.TGRC	// buffer for TGRA3 (semiper + deadtime)
#define	BUF_U			MTU3.TGRD	// buffer for TGRB3 (duty U)
#define	BUF_V			MTU4.TGRC	// buffer for TGRA4 (duty V)
#define	BUF_W			MTU4.TGRD	// buffer for TGRB4 (duty W) (LAST TO BE WRITTEN!)
#define SET_PWM(X, Y, Z)	{ BUF_U = ( X ); BUF_V = ( Y ); BUF_W = ( Z ); }	

// hardware related defines: inputs
#define KEY_ON			(0 == PORT1.PIDR.BIT.B4)	// PIN16 P14 IN  KEY
#define KEY_OFF			(0 != PORT1.PIDR.BIT.B4)	// PIN16 P14 IN  KEY
#define EXTSUP_ON		(0 == PORT3.PIDR.BIT.B5)	// PIN07 P35 NMI IN	EXTSUP
#define EXTSUP_OFF		(0 != PORT3.PIDR.BIT.B5)	// PIN07 P35 NMI IN	EXTSUP

// hardware related defines: outputs
#define SET_DL3			PORTE.PODR.BIT.B7 = 1;		// PIN40 PE7 OUT DL3
#define CLR_DL3			PORTE.PODR.BIT.B7 = 0;		// PIN40 PE7 OUT DL3
#define SET_DL2			PORTE.PODR.BIT.B0 = 1;		// PIN39 PE0 OUT DL2
#define CLR_DL2			PORTE.PODR.BIT.B0 = 0;		// PIN39 PE0 OUT DL2
#define SET_LIMOFF		PORT1.PODR.BIT.B7 = 1;		// PIN13 P17 OUT LIMOFF
#define RES_LIMOFF		PORT1.PODR.BIT.B7 = 0;		// PIN13 P17 OUT LIMOFF
#define DL3_ON			(0 != PORTE.PODR.BIT.B7)	// PIN40 PE7 OUT DL3
#define DL3_OFF			(0 == PORTE.PODR.BIT.B7)	// PIN40 PE7 OUT DL3
#define DL2_ON			(0 != PORTE.PODR.BIT.B0)	// PIN39 PE0 OUT DL2
#define DL2_OFF			(0 == PORTE.PODR.BIT.B0)	// PIN39 PE0 OUT DL2
#define LIMOFF_ON		(0 != PORT1.PODR.BIT.B7)	// PIN13 P17 OUT LIMOFF
#define LIMOFF_OFF		(0 == PORT1.PODR.BIT.B7)	// PIN13 P17 OUT LIMOFF

// global flags (global variables defined in "globvar.h" are used)
#define	TRGED			(0 != flg0.bit.b0)
#define	RBUSY			(0 != flg0.bit.b1)
#define	CSELD			(0 != flg0.bit.b2)
#define	FREE_03			(0 != flg0.bit.b3)
#define	FREE_04			(0 != flg0.bit.b4)
#define	FREE_05			(0 != flg0.bit.b5)
#define	FREE_06			(0 != flg0.bit.b6)
#define	FREE_07			(0 != flg0.bit.b7)

#define	NOT_TRGED		(0 == flg0.bit.b0)
#define	NOT_RBUSY		(0 == flg0.bit.b1)
#define	NOT_CSELD		(0 == flg0.bit.b2)
#define	NOT_FREE_03		(0 == flg0.bit.b3)
#define	NOT_FREE_04		(0 == flg0.bit.b4)
#define	NOT_FREE_05		(0 == flg0.bit.b5)
#define	NOT_FREE_06		(0 == flg0.bit.b6)
#define	NOT_FREE_07		(0 == flg0.bit.b7)

#define	SET_TRGED		flg0.bit.b0 = 1;
#define	SET_RBUSY		flg0.bit.b1 = 1;
#define	SET_CSELD		flg0.bit.b2 = 1;
#define	SET_FREE_03		flg0.bit.b3 = 1;
#define	SET_FREE_04		flg0.bit.b4 = 1;
#define	SET_FREE_05		flg0.bit.b5 = 1;
#define	SET_FREE_06		flg0.bit.b6 = 1;
#define	SET_FREE_07		flg0.bit.b7 = 1;

#define	RES_TRGED		flg0.bit.b0 = 0;
#define	RES_RBUSY		flg0.bit.b1 = 0;
#define	RES_CSELD		flg0.bit.b2 = 0;
#define	RES_FREE_03		flg0.bit.b3 = 0;
#define	RES_FREE_04		flg0.bit.b4 = 0;
#define	RES_FREE_05		flg0.bit.b5 = 0;
#define	RES_FREE_06		flg0.bit.b6 = 0;
#define	RES_FREE_07		flg0.bit.b7 = 0;

#define	FREE_10			(0 != flg1.bit.b0)
#define	FREE_11			(0 != flg1.bit.b1)
#define	FREE_12			(0 != flg1.bit.b2)
#define	FREE_13			(0 != flg1.bit.b3)
#define	FREE_14			(0 != flg1.bit.b4)
#define	FREE_15			(0 != flg1.bit.b5)
#define	FREE_16			(0 != flg1.bit.b6)
#define	FREE_17			(0 != flg1.bit.b7)

#define	NOT_FREE_10		(0 == flg1.bit.b0)
#define	NOT_FREE_11		(0 == flg1.bit.b1)
#define	NOT_FREE_12		(0 == flg1.bit.b2)
#define	NOT_FREE_13		(0 == flg1.bit.b3)
#define	NOT_FREE_14		(0 == flg1.bit.b4)
#define	NOT_FREE_15		(0 == flg1.bit.b5)
#define	NOT_FREE_16		(0 == flg1.bit.b6)
#define	NOT_FREE_17		(0 == flg1.bit.b7)

#define	SET_FREE_10		flg1.bit.b0 = 1;
#define	SET_FREE_11		flg1.bit.b1 = 1;
#define	SET_FREE_12		flg1.bit.b2 = 1;
#define	SET_FREE_13		flg1.bit.b3 = 1;
#define	SET_FREE_14		flg1.bit.b4 = 1;
#define	SET_FREE_15		flg1.bit.b5 = 1;
#define	SET_FREE_16		flg1.bit.b6 = 1;
#define	SET_FREE_17		flg1.bit.b7 = 1;

#define	RES_FREE_10		flg1.bit.b0 = 0;
#define	RES_FREE_11		flg1.bit.b1 = 0;
#define	RES_FREE_12		flg1.bit.b2 = 0;
#define	RES_FREE_13		flg1.bit.b3 = 0;
#define	RES_FREE_14		flg1.bit.b4 = 0;
#define	RES_FREE_15		flg1.bit.b5 = 0;
#define	RES_FREE_16		flg1.bit.b6 = 0;
#define	RES_FREE_17		flg1.bit.b7 = 0;

#endif	// _GLOBDEF_H

