
/*******************************************************************************
*	DISCLAIMER
*
*	THIS SOFTWARE IS PROVIDED "AS IS" AND BFG ENGINEERING MAKES NO WARRANTIES
*	REGARDING THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING
*	BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
*	PURPOSE AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
*
*	WE (BFG ENGINEERING) DO NOT WARRANT THAT THIS SOFTWARE IS FREE FROM CLAIMS
*	BY A THIRD PARTY OF COPYRIGHT, PATENT, TRADEMARK, TRADE SECRET OR ANY OTHER
*	INTELLECTUAL PROPERTY INFRINGIMENT.
*
*	TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, BFG ENGINEERING
*	SHALL NOT BE LIABLE FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR
*	CONSEQUENTIAL DAMAGES FOR ANY REASON RELATED TO THIS SOFTWARE, EVEN IF BFG
*	ENGINEERING HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
*	PARTICULARLY, UNDER NO CIRCUMSTANCES ARE WE LIABLE FOR ANY OF THE FOLLOWING:
*	- THIRD PARTY CLAIMS AGAINST YOU FOR LOSSES OR DAMAGES;
*	- LOSS OF, OR DAMAGE TO YOUR RECORDS OR DATA; OR
*	- ECONOMIC CONSEQUENTIAL DAMAGES (INCLUDING LOST PROFITS OR SAVINGS) OR
*	  INCIDENTAL DAMAGES, EVEN IF WE ARE INFORMED OF THEIR POSSIBILITY.
*
*	BFG ENGINEERING DOES NOT WARRANT UNINTERRUPTED OR ERROR_FREE OPERATION OF
*	THIS SOFTWARE.
*	BFG ENGINEERING HAS NO OBLIGATION TO PROVIDE SERVICE, DEFECT CORRECTION OR
*	ANY MANTEINANCE OF THIS SOFTWARE; OR TO SUPPLY ANY UPDATES OR ENHANCEMENTS
*	EVEN IF SUCH ARE, OR LATER BECOME, AVAILABLE.
*
*	IF YOU DOWNLOAD OR USE THIS SOFTWARE YOU AGREE TO THESE TERMS, AND DO IT AT
*	YOUR OWN RISK.
*******************************************************************************/

/*******************************************************************************
*	File:		hardware_wsetup.c
*	Project:	YRotateItRX111_3s_V1
*	Version:	1.0
*	Date:		01-07-2014 (DD-MM-YYYY)
*	Author:		Andrea Ghirardello
*	Contents:	Hardware setup file
*******************************************************************************/

#include "globdef.h"

/* PIN DESCRIPTION YRMCKITRX111 ************************************************

 1 P27			EEPSCL	OUT
 2 P26			EEPSDA	I/O
 3 MD			mode
 4 RES			reset
 5 XCOUT		xcout
 6 PH7 XCIN		xcin
 7 P35 NMI		EXTSUP	IN
 8 XTAL			xtal
 9 EXTAL		extal
10 VCL
11 VSS
12 VCC
>
13 P17			LIMOFF	OUT
14 P16 TXD1		TX		OUT
15 P15 RXD1		RX		IN
16 P14			KEY		IN
17 VCC_USB
18 USB0_DM
19 USB0_DP
20 VSS_USB
21 PC7 TCKLB	ENCB	IN
22 PC6 TCKLA	ENCA	IN
23 PC5 MTIOC3B	PWM_UH	OUT
24 PC4 MTIOC3D	PWM_UL	OUT
>
25 PB5 POE0		OVC		IN
26 PB3 MTIOC4A	PWM_VH	OUT
27 PB1 MTIOC4C	PWM_VL	OUT
28 VCC
29 PB0 MTIC5W	HALLW	IN
30 VSS
31 PA6 MTIC5V	HALLV	IN
32 PA4 MTIC5U	HALLU	IN
33 PA3 IRQ6		ENCZ	IN
34 PA1 MTIOC0B	ANOUT3	OUT
35 PE4 MTIOC4D	PWM_WL	OUT
36 PE3 MTIOC4B	PWM_WH	OUT
>
37 PE2 AN10		AUX		IN
38 PE1 AN9		TMP		IN
39 PE0			DL2		OUT
40 PE7			DL3		OUT
41 P46 AN6		VB		IN
42 P42 AN2		IW		IN
43 P41 AN1		IV		IN
44 PJ7 VREFL	DIGIO2	IN
45 P40 AN0		IU		IN
46 PJ6 VREFH	DIGIO1	IN
47 AVCC
48 AVSS

*******************************************************************************/

/*******************************************************************************
Macro definitions
*******************************************************************************/

/* Peripheral Module Clock D (PCLKD) */
#define _00000000_CGC_PCLKD_DIV_1           (0x00000000UL) /* x1 */
#define _00000001_CGC_PCLKD_DIV_2           (0x00000001UL) /* x1/2 */
#define _00000002_CGC_PCLKD_DIV_4           (0x00000002UL) /* x1/4 */
#define _00000003_CGC_PCLKD_DIV_8           (0x00000003UL) /* x1/8 */
#define _00000004_CGC_PCLKD_DIV_16          (0x00000004UL) /* x1/16 */
#define _00000005_CGC_PCLKD_DIV_32          (0x00000005UL) /* x1/32 */
#define _00000006_CGC_PCLKD_DIV_64          (0x00000006UL) /* x1/64 */
/* Peripheral Module Clock B (PCLKB) */
#define _00000000_CGC_PCLKB_DIV_1           (0x00000000UL) /* x1 */
#define _00000100_CGC_PCLKB_DIV_2           (0x00000100UL) /* x1/2 */
#define _00000200_CGC_PCLKB_DIV_4           (0x00000200UL) /* x1/4 */
#define _00000300_CGC_PCLKB_DIV_8           (0x00000300UL) /* x1/8 */
#define _00000400_CGC_PCLKB_DIV_16          (0x00000400UL) /* x1/16 */
#define _00000500_CGC_PCLKB_DIV_32          (0x00000500UL) /* x1/32 */
#define _00000600_CGC_PCLKB_DIV_64          (0x00000600UL) /* x1/64 */
/* System Clock (ICLK) */
#define _00000000_CGC_ICLK_DIV_1            (0x00000000UL) /* x1 */
#define _01000000_CGC_ICLK_DIV_2            (0x01000000UL) /* x1/2 */
#define _02000000_CGC_ICLK_DIV_4            (0x02000000UL) /* x1/4 */
#define _03000000_CGC_ICLK_DIV_8            (0x03000000UL) /* x1/8 */
#define _04000000_CGC_ICLK_DIV_16           (0x04000000UL) /* x1/16 */
#define _05000000_CGC_ICLK_DIV_32           (0x05000000UL) /* x1/32 */
#define _06000000_CGC_ICLK_DIV_64           (0x06000000UL) /* x1/64 */
/* System Clock (FCLK) */
#define _00000000_CGC_FCLK_DIV_1            (0x00000000UL) /* x1 */
#define _10000000_CGC_FCLK_DIV_2            (0x10000000UL) /* x1/2 */
#define _20000000_CGC_FCLK_DIV_4            (0x20000000UL) /* x1/4 */
#define _30000000_CGC_FCLK_DIV_8            (0x30000000UL) /* x1/8 */
#define _40000000_CGC_FCLK_DIV_16           (0x40000000UL) /* x1/16 */
#define _50000000_CGC_FCLK_DIV_32           (0x50000000UL) /* x1/32 */
#define _60000000_CGC_FCLK_DIV_64           (0x60000000UL) /* x1/64 */

/* System Clock Control Register 3 (SCKCR3) */
#define _0000_CGC_CLOCKSOURCE_LOCO          (0x0000U) /* LOCO */
#define _0100_CGC_CLOCKSOURCE_HOCO          (0x0100U) /* HOCO */
#define _0200_CGC_CLOCKSOURCE_MAINCLK       (0x0200U) /* Main clock oscillator */
#define _0300_CGC_CLOCKSOURCE_SUBCLK        (0x0300U) /* Sub-clock oscillator */
#define _0400_CGC_CLOCKSOURCE_PLL           (0x0400U) /* PLL circuit */

/* PLL Control Register (PLLCR) */
/* PLL Input Frequency Division Ratio Select (PLIDIV[1:0]) */
#define _0000_CGC_PLL_FREQ_DIV_1            (0x0000U) /* x1 */
#define _0001_CGC_PLL_FREQ_DIV_2            (0x0001U) /* x1/2 */
#define _0002_CGC_PLL_FREQ_DIV_4            (0x0002U) /* x1/4 */
/* Frequency Multiplication Factor Select (STC[5:0]) */
#define _0B00_CGC_PLL_FREQ_MUL_6            (0x0B00U) /* x6 */
#define _0F00_CGC_PLL_FREQ_MUL_8            (0x0F00U) /* x8 */

/* Oscillation Stop Detection Control Register (OSTDCR) */
/* Oscillation Stop Detection Interrupt Enable (OSTDIE) */
#define _00_CGC_OSC_STOP_INT_DISABLE        (0x00U) /* The oscillation stop detection interrupt is disabled */
#define _01_CGC_OSC_STOP_INT_ENABLE         (0x01U) /* The oscillation stop detection interrupt is enabled */
/* Oscillation Stop Detection Function Enable (OSTDE) */
#define _00_CGC_OSC_STOP_DISABLE            (0x00U) /* Oscillation stop detection function is disabled */
#define _80_CGC_OSC_STOP_ENABLE             (0x80U) /* Oscillation stop detection function is enabled */

/* Main Clock Oscillator Wait Control Register (MOSCWTCR) */
/* Main Clock Oscillator Wait Time (MSTS[4:0]) */
#define _00_CGC_OSC_WAIT_CYCLE_2            (0x00U) /* Wait time = 2 cycles */
#define _01_CGC_OSC_WAIT_CYCLE_1024         (0x01U) /* Wait time = 1024 cycles */
#define _02_CGC_OSC_WAIT_CYCLE_2048         (0x02U) /* Wait time = 2048 cycles */
#define _03_CGC_OSC_WAIT_CYCLE_4096         (0x03U) /* Wait time = 4096 cycles */
#define _04_CGC_OSC_WAIT_CYCLE_8192         (0x04U) /* Wait time = 8192 cycles */
#define _05_CGC_OSC_WAIT_CYCLE_16384        (0x05U) /* Wait time = 16384 cycles */
#define _06_CGC_OSC_WAIT_CYCLE_32768        (0x06U) /* Wait time = 32768 cycles */
#define _07_CGC_OSC_WAIT_CYCLE_65536        (0x07U) /* Wait time = 65536 cycles */

/* HOCO Wait Control Register (HOCOWTCR) */
/* HOCO Wait Time (HOCOWTCR) */
#define _05_CGC_HOCO_WAIT_CYCLE_138         (0x05U) /* Wait time = 138 cycles (34.5us) */
#define _06_CGC_HOCO_WAIT_CYCLE_266         (0x06U) /* Wait time = 266 cycles (66.5us) */

/* Port Switching Register B (PSRB) */
/* PB0/PC0 Switching (PSEL0) */
#define _00_PORT_PSEL0_PB0      			(0x00U) /* PB0 general I/O port function is selected */
#define _01_PORT_PSEL0_PC0      			(0x01U) /* PC0 general I/O port function is selected */
/* PB1/PC1 Switching (PSEL1) */
#define _00_PORT_PSEL1_PB1      			(0x00U) /* PB1 general I/O port function is selected */
#define _02_PORT_PSEL1_PC1      			(0x02U) /* PC1 general I/O port function is selected */
/* PB3 Switching (PSEL3) */
#define _00_PORT_PSEL3_PB3      			(0x00U) /* PB3 general I/O port function is selected */
/* PB5/PC3 Switching (PSEL5) */
#define _00_PORT_PSEL3_PB5      			(0x00U) /* PB5 general I/O port function is selected */
#define _20_PORT_PSEL3_PC3      			(0x20U) /* PC3 general I/O port function is selected */

/* non-existent pins */
#define _28_PDR0_DEFAULT 					(0x28U) 	// nonexistent pins: P03, P05
#define _07_PDR3_DEFAULT 					(0x07U) 	// nonexistent pins: P30, P31, P32
#define _18_PDR4_DEFAULT 					(0x18U) 	// nonexistent pins: P43, P44
#define _30_PDR5_DEFAULT 					(0x30U) 	// nonexistent pins: P54, P55
#define _01_PDRA_DEFAULT 					(0x01U) 	// nonexistent pins: PA0
#define _C0_PDRB_DEFAULT 					(0xC0U) 	// nonexistent pins: PB6, PB7 (depending on PSRB)
#define _0C_PDRC_DEFAULT 					(0x0CU) 	// nonexistent pins: PC0, PC1, PC2, PC3 (depending on PSRB)
#define _60_PDRE_DEFAULT 					(0x60U) 	// nonexistent pins: PE5, PE6


void HardwareSetup(void)
{
	uint32_t sckcr_dummy;

    // Enable writing to registers related to operating modes, LPC, CGC and software reset
    SYSTEM.PRCR.WORD = 0xA507U; 

    // Enable writing to MPC pin function control registers
    MPC.PWPR.BIT.B0WI = 0U;    
    MPC.PWPR.BIT.PFSWE = 1U;

    /******************* Clock ******************/

    // Set system clock
    //SYSTEM.SCKCR.LONG = _00000000_CGC_PCLKD_DIV_1 | _00000000_CGC_PCLKB_DIV_1 | _00000000_CGC_ICLK_DIV_1 |
    //                    _00000000_CGC_FCLK_DIV_1;
    sckcr_dummy = 0x00000000UL;
    SYSTEM.SCKCR.LONG = sckcr_dummy;
    while(SYSTEM.SCKCR.LONG != sckcr_dummy);

    // Disable sub-clock
    SYSTEM.SOSCCR.BIT.SOSTP = 1U;
    while(1U != SYSTEM.SOSCCR.BIT.SOSTP);

    // Disable sub-clock
    RTC.RCR3.BIT.RTCEN = 0U;
	while(0U != RTC.RCR3.BIT.RTCEN);

	// Set HOCO
	SYSTEM.HOCOCR.BIT.HCSTP = 0U;
    while(1U != SYSTEM.OSCOVFSR.BIT.HCOVF);				// Wait for HOCO wait counter overflow
    SYSTEM.HOCOWTCR.BYTE = _06_CGC_HOCO_WAIT_CYCLE_266;	// Set HOCO wait time

    // Set clock source
    SYSTEM.SCKCR3.WORD = _0100_CGC_CLOCKSOURCE_HOCO;
    while(SYSTEM.SCKCR3.WORD != _0100_CGC_CLOCKSOURCE_HOCO);

    /******************* Clock end *****************/

	/******************* Ports *********************/

	// PORT B/C SWITCHING	*******

    //PORT.PSRA.BYTE = 0x00;		// ONLY FOR 64pin VERSION
    //PORT.PSRA.BIT.PSEL7 = 0;		// B7	0 -> PIN27 is PB7	1 -> PIN27 is PC1
    //PORT.PSRA.BIT.PSEL6 = 0;		// B6	0 -> PIN29 is PB6	1 -> PIN29 is PC0
    	// B5, .., B0

    PORT.PSRB.BYTE = 0x00;			// ONLY FOR 48pin VERSION
    	// B7, B6
    //PORT.PSRB.BIT.PSEL5 = 0;		// B5	0 -> PIN25 is PB5	1 -> PIN25 is PC3
    	// B4
    //PORT.PSRB.BIT.PSEL3 = 0;		// B3	0 -> PIN26 is PB3	1 -> PIN26 is PC2
    	// B2
    //PORT.PSRB.BIT.PSEL1 = 0;		// B1	0 -> PIN27 is PB1	1 -> PIN27 is PC1
    //PORT.PSRB.BIT.PSEL0 = 0;		// B0	0 -> PIN29 is PB0	1 -> PIN29 is PC0


    // PORT0	********************

    PORT0.PDR.BYTE = _28_PDR0_DEFAULT;	// nonexistent pins init as output
	PORT0.PODR.BYTE = 0x00;


	// PORT1	********************

    PORT1.PDR.BYTE = 0xC0;			// port direction register
    //PORT1.PDR.BIT.B7 = 1;			// PIN13 P17		OUT	LIMOFF
    //PORT1.PDR.BIT.B6 = 1;			// PIN14 P16 TXD1	OUT	TX
    //PORT1.PDR.BIT.B5 = 0;			// PIN15 P15 RXD1	IN	RX
    //PORT1.PDR.BIT.B4 = 0;			// PIN16 P14		IN	KEY
    	// B3, .., B0

    PORT1.PODR.BYTE = 0x40;			// port output data register
    //PORT1.PODR.BIT.B7 = 0;		// PIN13 P17		OUT	LIMOFF
    //PORT1.PODR.BIT.B6 = 1;		// PIN14 P16 TXD1	OUT TX
    //PORT1.PODR.BIT.B5 = 0;		// PIN15 P15 RXD1	IN  RX
    //PORT1.PODR.BIT.B4 = 0;		// PIN16 P14		IN  KEY
    	// B3, .., B0

    //PORT1.PIDR.BYTE				// port input data register
    //PORT1.PIDR.BIT.B7				// PIN13 P17		OUT	LIMOFF
    //PORT1.PIDR.BIT.B6				// PIN14 P16 TXD1	OUT TX
    //PORT1.PIDR.BIT.B5				// PIN15 P15 RXD1	IN  RX
    //PORT1.PIDR.BIT.B4				// PIN16 P14		IN  KEY
    	// B3, .., B0

    //PORT1.PMR.BYTE = 0x00;		// port mode register
    //PORT1.PMR.BIT.B7 = 0;			// PIN13 P17		OUT	LIMOFF
    //PORT1.PMR.BIT.B6 = 0;			// PIN14 P16 TXD1	OUT TX
    //PORT1.PMR.BIT.B5 = 0;			// PIN15 P15 RXD1	IN  RX
    //PORT1.PMR.BIT.B4 = 0;			// PIN16 P14		IN  KEY
    	// B3, .., B0

    //PORT1.ODR1.BYTE = 0x00;		// open drain control register 1
    //PORT1.PCR.BYTE = 0x00;		// pull up control register

    // PORT2	********************

    PORT2.PDR.BYTE = 0x80;			// port direction register
    //PORT2.PDR.BIT.B7 = 1;			// PIN01 P27	OUT EEPSCL
    //PORT2.PDR.BIT.B6 = 0;			// PIN02 P26	I/O EEPSDA
    	// B5, .., B0

    PORT2.PODR.BYTE = 0xC0;			// port output data register
    //PORT2.PODR.BIT.B7 = 1;		// PIN01 P27	OUT EEPSCL
    //PORT2.PODR.BIT.B6 = 1;		// PIN02 P26	I/O EEPSDA
    	// B5, .., B0

    //PORT2.PIDR.BYTE				// port input data register
    //PORT2.PIDR.BIT.B7				// PIN01 P27	OUT EEPSCL
    //PORT2.PIDR.BIT.B6				// PIN02 P26	I/O EEPSDA
    	// B5, .., B0

    //PORT2.PMR.BYTE = 0x00;		// port mode register
    //PORT2.PMR.BIT.B7 = 0;			// PIN01 P27	OUT EEPSCL
    //PORT2.PMR.BIT.B6 = 0;			// PIN02 P26	I/O EEPSDA
    	// B5, .., B0

    PORT2.ODR1.BYTE = 0x10;			// open drain control register 1
    	// B7
	//PORT2.ODR1.BIT.B6 = 0;		// PIN01 P27 OUT EEPSCL 0 -> cmos output
    	// B5
	//PORT2.ODR1.BIT.B4 = 1;		// PIN02 P26 I/O EEPSDA 1 -> open drain output
    	// B3, .., B0

    //PORT2.PCR.BYTE = 0x00;		// pull up control register

    // PORT3	********************

	PORT3.PDR.BYTE = _07_PDR3_DEFAULT;	// nonexistent pins init as output
	PORT3.PODR.BYTE = 0x00;

    //PORT3.PIDR.BYTE				// port input data register
    	// B7, .., B3
    //PORT3.PIDR.BIT.B5				// PIN07 P35 NMI	IN	EXTSUP
    	// B4, .., B0

	// PORT4	********************

	PORT4.PDR.BYTE = _18_PDR4_DEFAULT;	// nonexistent pins init as output
	//PORT4.PDR.BYTE |= 0x00;		// port direction register
		// B7
	//PORT4.PDR.BIT.B6 = 0;			// PIN41 P46 AN6	IN  VB
		// B5, .., B3
	//PORT4.PDR.BIT.B2 = 0;			// PIN42 P42 AN2	IN  IW
	//PORT4.PDR.BIT.B1 = 0;			// PIN43 P41 AN1	IN  IV
	//PORT4.PDR.BIT.B0 = 0;			// PIN45 P40 AN0	IN  IU

	PORT4.PODR.BYTE = 0x00;			// port output data register
		// B7
	//PORT4.PODR.BIT.B6 = 0;		// PIN41 P46 AN6	IN  VB
		// B5, .., B3
	//PORT4.PODR.BIT.B2 = 0;		// PIN42 P42 AN2	IN  IW
	//PORT4.PODR.BIT.B1 = 0;		// PIN43 P41 AN1	IN  IV
	//PORT4.PODR.BIT.B0 = 0;		// PIN45 P40 AN0	IN  IU

	//PORT4.PIDR.BYTE				// port input data register
		// B7
	//PORT4.PIDR.BIT.B6				// PIN41 P46 AN6	IN  VB
		// B5, .., B3
	//PORT4.PIDR.BIT.B2				// PIN42 P42 AN2	IN  IW
	//PORT4.PIDR.BIT.B1				// PIN43 P41 AN1	IN  IV
	//PORT4.PIDR.BIT.B0				// PIN45 P40 AN0	IN  IU

	//PORT4.PMR.BYTE = 0x00;		// port mode register
		// B7
	//PORT4.PMR.BIT.B6 = 0;			// PIN41 P46 AN6	IN  VB
		// B5, .., B3
	//PORT4.PMR.BIT.B2 = 0;			// PIN42 P42 AN2	IN  IW
	//PORT4.PMR.BIT.B1 = 0;			// PIN43 P41 AN1	IN  IV
	//PORT4.PMR.BIT.B0 = 0;			// PIN45 P40 AN0	IN  IU

	//PORT4.PCR.BYTE = 0x00;		// pull up control register


	// PORT5	********************

	PORT5.PDR.BYTE = _30_PDR5_DEFAULT;	// nonexistent pins init as output
	PORT5.PODR.BYTE = 0x00;


	// PORTA	********************

    PORTA.PDR.BYTE = _01_PDRA_DEFAULT;	// nonexistent pins init as output
	PORTA.PDR.BYTE |= 0x02;			// port direction register
		// B7
	//PORTA.PDR.BIT.B6 = 0;			// PIN31 PA6 MTIC5V		IN  HALLV
		// B5
	//PORTA.PDR.BIT.B4 = 0;			// PIN32 PA4 MTIC5U		IN  HALLU
	//PORTA.PDR.BIT.B3 = 0;			// PIN33 PA3 IRQ6		IN  ENCZ
		// B2
	//PORTA.PDR.BIT.B1 = 1;			// PIN34 PA1 MTIOC0B	OUT ANOUT3
		// B0

	PORTA.PODR.BYTE = 0x00;			// port output data register
		// B7
	//PORTA.PODRBIT.B6 = 0;			// PIN31 PA6 MTIC5V		IN  HALLV
		// B5
	//PORTA.PODR.BIT.B4 = 0;		// PIN32 PA4 MTIC5U		IN  HALLU
	//PORTA.PODR.BIT.B3 = 0;		// PIN33 PA3 IRQ6		IN  ENCZ
		// B2
	//PORTA.PODR.BIT.B1 = 0;		// PIN34 PA1 MTIOC0B	OUT ANOUT3
		// B0

	//PORTA.PIDR.BYTE				// port input data register
		// B7
	//PORTA.PIDR.BIT.B6				// PIN31 PA6 MTIC5V		IN  HALLV
		// B5
	//PORTA.PIDR.BIT.B4				// PIN32 PA4 MTIC5U		IN  HALLU
	//PORTA.PIDR.BIT.B3				// PIN33 PA3 IRQ6		IN  ENCZ
		// B2
	//PORTA.PIDR.BIT.B1				// PIN34 PA1 MTIOC0B	OUT ANOUT3
		// B0

	//PORTA.PMR.BYTE |= 0x00;		// port mode register
		// B7
	//PORTA.PMR.BIT.B6 = 0;			// PIN31 PA6 MTIC5V		IN  HALLV
		// B5
	//PORTA.PMR.BIT.B4 = 0;			// PIN32 PA4 MTIC5U		IN  HALLU
	//PORTA.PMR.BIT.B3 = 0;			// PIN33 PA3 IRQ6		IN  ENCZ
		// B2
	//PORTA.PMR.BIT.B1 = 0;			// PIN34 PA1 MTIOC0B	OUT ANOUT3
		// B0

	//PORTA.ODR0.BYTE |= 0x00;		// open drain control register 0
	//PORTA.ODR1.BYTE |= 0x00;		// open drain control register 1
	//PORTA.PCR.BYTE |= 0x00;		// pull up control register


	// PORTB	********************
	// WARNING: PWM OUTPUTS ARE HIGH ACTIVE!

	PORTB.PDR.BYTE = _C0_PDRB_DEFAULT;	// nonexistent pins init as output
	PORTB.PODR.BYTE = 0x00;			// port output data register
	PORTB.PDR.BYTE |= 0x0A;			// port direction register
		// B7, B6
	//PORTB.PDR.BIT.B5 = 0;			// PIN25 PB5 (PC3) POE0		IN  OVC
		// B4
	//PORTB.PDR.BIT.B3 = 1;			// PIN26 PB3 (PC2) MTIOC4A	OUT PWM_VH
		// B2
	//PORTB.PDR.BIT.B1 = 1;			// PIN27 PB1 (PC1) MTIOC4C	OUT PWM_VL
	//PORTB.PDR.BIT.B0 = 0;			// PIN29 PB0 (PC0) MTIC5W	IN  HALLW

	PORTB.PODR.BYTE = 0x00;			// port output data register
		// B7, B6
	//PORTB.PODR.BIT.B5 = 0;		// PIN25 PB5 (PC3) POE0		IN  OVC
		// B4
	//PORTB.PODR.BIT.B3 = 0;		// PIN26 PB3 (PC2) MTIOC4A	OUT PWM_VH
		// B2
	//PORTB.PODR.BIT.B1 = 0;		// PIN27 PB1 (PC1) MTIOC4C	OUT PWM_VL
	//PORTB.PODR.BIT.B0 = 0;		// PIN29 PB0 (PC0) MTIC5W	IN  HALLW

	//PORTB.PIDR.BYTE				// port input data register
		// B7, B6
	//PORTB.PIDR.BIT.B5				// PIN25 PB5 (PC3) POE0		IN  OVC
		// B4
	//PORTB.PIDR.BIT.B3				// PIN26 PB3 (PC2) MTIOC4A	OUT PWM_VH
		// B2
	//PORTB.PIDR.BIT.B1				// PIN27 PB1 (PC1) MTIOC4C	OUT PWM_VL
	//PORTB.PIDR.BIT.B0				// PIN29 PB0 (PC0) MTIC5W	IN  HALLW

	//PORTB.PMR.BYTE = 0x00;		// port mode register
		// B7, B6
	//PORTB.PMR.BIT.B5 = 0;			// PIN25 PB5 (PC3) POE0		IN  OVC
		// B4
	//PORTB.PMR.BIT.B3 = 0;			// PIN26 PB3 (PC2) MTIOC4A	OUT PWM_VH
		// B2
	//PORTB.PMR.BIT.B1 = 0;			// PIN27 PB1 (PC1) MTIOC4C	OUT PWM_VL
	//PORTB.PMR.BIT.B0 = 0;			// PIN29 PB0 (PC0) MTIC5W	IN  HALLW

	//PORTB.ODR0.BYTE = 0x00;		// open drain control register 0
	//PORTB.ODR1.BYTE = 0x00;		// open drain control register 1
	//PORTB.PCR.BYTE = 0x00;		// pull up control register
	//PORTB.DSCR.BYTE = 0x00;		// drive capacity control register


	// PORTC	********************
	// WARNING: PWM OUTPUTS ARE HIGH ACTIVE!

	PORTC.PDR.BYTE = _0C_PDRC_DEFAULT;	// nonexistent pins init as output
	PORTC.PODR.BYTE = 0x00;			// port output data register
	PORTC.PDR.BYTE |= 0x30;			// port direction register
	//PORTC.PDR.BIT.B7 = 0;			// PIN21 PC7 TCKLB			IN  ENCB
	//PORTC.PDR.BIT.B6 = 0;			// PIN22 PC6 TCKLA			IN  ENCA
	//PORTC.PDR.BIT.B5 = 1;			// PIN23 PC5 MTIOC3B		OUT PWM_UH
	//PORTC.PDR.BIT.B4 = 1;			// PIN24 PC4 MTIOC3D		OUT PWM_UL
	//PORTC.PDR.BIT.B3 = 0;			// PIN25 PB5 (PC3) POE0		IN  OVC
	//PORTC.PDR.BIT.B2 = 0;			// PIN26 PB3 (PC2) MTIOC4A	OUT PWM_VH
	//PORTC.PDR.BIT.B1 = 0;			// PIN27 PB1 (PC1) MTIOC4C	OUT PWM_VL
	//PORTC.PDR.BIT.B0 = 0;			// PIN29 PB0 (PC0) MTIC5W	IN  HALLW

	PORTC.PODR.BYTE = 0x00;			// port output data register
	//PORTC.PODR.BIT.B7 = 0;		// PIN21 PC7 TCKLB			IN  ENCB
	//PORTC.PODR.BIT.B6 = 0;		// PIN22 PC6 TCKLA			IN  ENCA
	//PORTC.PODR.BIT.B5 = 0;		// PIN23 PC5 MTIOC3B		OUT PWM_UH
	//PORTC.PODR.BIT.B4 = 0;		// PIN24 PC4 MTIOC3D		OUT PWM_UL
	//PORTC.PODR.BIT.B3 = 0;		// PIN25 PB5 (PC3) POE0		IN  OVC
	//PORTC.PODR.BIT.B2 = 0;		// PIN26 PB3 (PC2) MTIOC4A	OUT PWM_VH
	//PORTC.PODR.BIT.B1 = 0;		// PIN27 PB1 (PC1) MTIOC4C	OUT PWM_VL
	//PORTC.PODR.BIT.B0 = 0;		// PIN29 PB0 (PC0) MTIC5W	IN  HALLW

	//PORTC.PIDR.BYTE				// port input data register
	//PORTC.PIDR.BIT.B7				// PIN21 PC7 TCKLB			IN  ENCB
	//PORTC.PIDR.BIT.B6				// PIN22 PC6 TCKLA			IN  ENCA
	//PORTC.PIDR.BIT.B5				// PIN23 PC5 MTIOC3B		OUT PWM_UH
	//PORTC.PIDR.BIT.B4				// PIN24 PC4 MTIOC3D		OUT PWM_UL
	//PORTC.PIDR.BIT.B3				// PIN25 PB5 (PC3) POE0		IN  OVC
	//PORTC.PIDR.BIT.B2				// PIN26 PB3 (PC2) MTIOC4A	OUT PWM_VH
	//PORTC.PIDR.BIT.B1				// PIN27 PB1 (PC1) MTIOC4C	OUT PWM_VL
	//PORTC.PIDR.BIT.B0				// PIN29 PB0 (PC0) MTIC5W	IN  HALLW

	//PORTC.PMR.BYTE =0x00;			// port mode register
	//PORTC.PMR.BIT.B7 = 0;			// PIN21 PC7 TCKLB			IN  ENCB
	//PORTC.PMR.BIT.B6 = 0;			// PIN22 PC6 TCKLA			IN  ENCA
	//PORTC.PMR.BIT.B5 = 0;			// PIN23 PC5 MTIOC3B		OUT PWM_UH
	//PORTC.PMR.BIT.B4 = 0;			// PIN24 PC4 MTIOC3D		OUT PWM_UL
	//PORTC.PMR.BIT.B3 = 0;			// PIN25 PB5 (PC3) POE0		IN  OVC
	//PORTC.PMR.BIT.B2 = 0;			// PIN26 PB3 (PC2) MTIOC4A	OUT PWM_VH
	//PORTC.PMR.BIT.B1 = 0;			// PIN27 PB1 (PC1) MTIOC4C	OUT PWM_VL
	//PORTC.PMR.BIT.B0 = 0;			// PIN29 PB0 (PC0) MTIC5W	IN  HALLW

	//PORTC.ODR0.BYTE = 0x00;		// open drain control register 0
	//PORTC.ODR1.BYTE = 0x00;		// open drain control register 1
	//PORTC.PCR.BYTE = 0x00;		// pull up control register
	//PORTC.DSCR.BYTE = 0x00;		// drive capacity control register


	// PORTE	********************
	// WARNING: PWM OUTPUTS ARE HIGH ACTIVE!

    PORTE.PDR.BYTE = _60_PDRE_DEFAULT;	// nonexistent pins init as output
	PORTE.PODR.BYTE = 0x00;			// port output data register
	PORTE.PDR.BYTE |= 0x99;			// port direction register
	//PORTE.PDR.BIT.B7 = 1;			// PIN40 PE7			OUT DL3
		// B6, B5
	//PORTE.PDR.BIT.B4 = 1;			// PIN35 PE4 MTIOC4D	OUT PWM_WL
	//PORTE.PDR.BIT.B3 = 1;			// PIN36 PE3 MTIOC4B	OUT PWM_WH
	//PORTE.PDR.BIT.B2 = 0;			// PIN37 PE2 AN10		IN  AUX
	//PORTE.PDR.BIT.B1 = 0;			// PIN38 PE1 AN9		IN  TMP
	//PORTE.PDR.BIT.B0 = 1;			// PIN39 PE0			OUT DL2

	PORTE.PODR.BYTE = 0x00;			// port output data register
	//PORTE.PODR.BIT.B7 = 0;		// PIN40 PE7			OUT DL3
		// B6, B5
	//PORTE.PODR.BIT.B4 = 0;		// PIN35 PE4 MTIOC4D	OUT PWM_WL
	//PORTE.PODR.BIT.B3 = 0;		// PIN36 PE3 MTIOC4B	OUT PWM_WH
	//PORTE.PODR.BIT.B2 = 0;		// PIN37 PE2 AN10		IN  AUX
	//PORTE.PODR.BIT.B1 = 0;		// PIN38 PE1 AN9		IN  TMP
	//PORTE.PODR.BIT.B0 = 0;		// PIN39 PE0			OUT DL2

	//PORTE.PIDR.BYTE				// port input data register
	//PORTE.PIDR.BIT.B7				// PIN40 PE7			OUT DL3
		// B6, B5
	//PORTE.PIDR.BIT.B4				// PIN35 PE4 MTIOC4D	OUT PWM_WL
	//PORTE.PIDR.BIT.B3				// PIN36 PE3 MTIOC4B	OUT PWM_WH
	//PORTE.PIDR.BIT.B2				// PIN37 PE2 AN10		IN  AUX
	//PORTE.PIDR.BIT.B1				// PIN38 PE1 AN9		IN  TMP
	//PORTE.PODR.BIT.B0				// PIN39 PE0			OUT DL2

	//PORTE.PMR.BYTE = 0x00;		// port mode register
	//PORTE.PMR.BIT.B7 = 0;			// PIN40 PE7			OUT DL3
		// B6, B5
	//PORTE.PMR.BIT.B4 = 0;			// PIN35 PE4 MTIOC4D	OUT PWM_WL
	//PORTE.PMR.BIT.B3 = 0;			// PIN36 PE3 MTIOC4B	OUT PWM_WH
	//PORTE.PMR.BIT.B2 = 0;			// PIN37 PE2 AN10		IN  AUX
	//PORTE.PMR.BIT.B1 = 0;			// PIN38 PE1 AN9		IN  TMP
	//PORTE.PMR.BIT.B0 = 0;			// PIN39 PE0			OUT DL2

	//PORTE.ODR0.BYTE = 0x00;		// open drain control register 0
	//PORTE.ODR1.BYTE = 0x00;		// open drain control register 1
	//PORTE.PCR.BYTE = 0x00;		// pull up control register

	
	// PORTH	********************

	//PORTH.PIDR.BYTE				// port input data register
	//PORTH.PIDR.BIT.B7				// PIN06 PH7 XCIN IN  NU
		// B6, .., B0

	//PORTH.PMR.BYTE = 0x00;		// port mode register
	//PORTH.PMR.BIT.B7 = 0;			// PIN06 PH7 XCIN IN  NU
		// B6, .., B0


	// PORTJ	********************

    //PORTJ.PDR.BYTE = 0x00;		// port direction register
    //PORTJ.PDR.BIT.B7 = 0;			// PIN44 PJ7 VREFL IN  DIGIO2
    //PORTJ.PDR.BIT.B6 = 0;			// PIN46 PJ6 VREFH IN  DIGIO1
    	// B5, .., B0

    //PORTJ.PODR.BYTE = 0x00;		// port output data register
    //PORTJ.PODR.BIT.B7 = 0;		// PIN44 PJ7 VREFL IN  DIGIO2
    //PORTJ.PODR.BIT.B6 = 0;		// PIN46 PJ6 VREFH IN  DIGIO1
    	// B5, .., B0

    //PORTJ.PIDR.BYTE				// port input data register
    //PORTJ.PIDR.BIT.B7				// PIN44 PJ7 VREFL IN  DIGIO2
    //PORTJ.PIDR.BIT.B6				// PIN46 PJ6 VREFH IN  DIGIO1
    	// B5, .., B0

    //PORTJ.PMR.BYTE = 0x00;		// port mode register
    //PORTJ.PMR.BIT.B7 = 0;			// PIN44 PJ7 VREFL IN  DIGIO2
    //PORTJ.PMR.BIT.B6 = 0;			// PIN46 PJ6 VREFH IN  DIGIO1
    	// B5, .., B0

	/******************* Ports end **********************/


    /* Disable writing to MPC pin function control registers */
    MPC.PWPR.BIT.PFSWE = 0U;    
    MPC.PWPR.BIT.B0WI = 1U;     

    /* Enable protection */
    SYSTEM.PRCR.WORD = 0xA500U;  

}
