
/*******************************************************************************
*	DISCLAIMER
*
*	THIS SOFTWARE IS PROVIDED "AS IS" AND BFG ENGINEERING MAKES NO WARRANTIES
*	REGARDING THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING
*	BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
*	PURPOSE AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
*
*	WE (BFG ENGINEERING) DO NOT WARRANT THAT THIS SOFTWARE IS FREE FROM CLAIMS
*	BY A THIRD PARTY OF COPYRIGHT, PATENT, TRADEMARK, TRADE SECRET OR ANY OTHER
*	INTELLECTUAL PROPERTY INFRINGIMENT.
*
*	TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, BFG ENGINEERING
*	SHALL NOT BE LIABLE FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR
*	CONSEQUENTIAL DAMAGES FOR ANY REASON RELATED TO THIS SOFTWARE, EVEN IF BFG
*	ENGINEERING HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
*	PARTICULARLY, UNDER NO CIRCUMSTANCES ARE WE LIABLE FOR ANY OF THE FOLLOWING:
*	- THIRD PARTY CLAIMS AGAINST YOU FOR LOSSES OR DAMAGES;
*	- LOSS OF, OR DAMAGE TO YOUR RECORDS OR DATA; OR
*	- ECONOMIC CONSEQUENTIAL DAMAGES (INCLUDING LOST PROFITS OR SAVINGS) OR
*	  INCIDENTAL DAMAGES, EVEN IF WE ARE INFORMED OF THEIR POSSIBILITY.
*
*	BFG ENGINEERING DOES NOT WARRANT UNINTERRUPTED OR ERROR_FREE OPERATION OF
*	THIS SOFTWARE.
*	BFG ENGINEERING HAS NO OBLIGATION TO PROVIDE SERVICE, DEFECT CORRECTION OR
*	ANY MANTEINANCE OF THIS SOFTWARE; OR TO SUPPLY ANY UPDATES OR ENHANCEMENTS
*	EVEN IF SUCH ARE, OR LATER BECOME, AVAILABLE.
*
*	IF YOU DOWNLOAD OR USE THIS SOFTWARE YOU AGREE TO THESE TERMS, AND DO IT AT
*	YOUR OWN RISK.
*******************************************************************************/

/*******************************************************************************
*	File:		motorcontrol.h
*	Project:	YRotateItRX111_3s_V1
*	Version:	1.0
*	Date:		01-07-2014 (DD-MM-YYYY)
*	Author:		Andrea Ghirardello
*	Contents:	Header for motorcontrol.c
*******************************************************************************/

#ifndef _MOTORCONTROL_H
#define _MOTORCONTROL_H

#include "typedefine.h"

/******************************************************************************
* Function Name: MC_WaitSync
* Description  : time synchronization control function
* Arguments    : none
* Return Value : 0 if main loop time is elapsed, 1 if not
******************************************************************************/
uint16_t MC_WaitSync(void);

/******************************************************************************
* Function Name: MC_StartDBPWM
* Description  : set and start debug pwm outputs
* Arguments    : none
* Return Value : none
******************************************************************************/
void MC_StartDBPWM(void);

/******************************************************************************
* Name:			MC_SetPwmConst
* Description:	calculation of constants related to sampling (and pwm) freq.
* Arguments:	desired sampling frequency in Hz sam_fre_Hz_req
*				desired pwm_freq/sam_freq ratio fre_ratio_req
* Return Value:	0 all OK, an error code if something wrong (clamp
*				performed because limits exceeded)
******************************************************************************/
uint16_t MC_SetPwmConst(uint16_t sam_fre_Hz_req, uint16_t fre_ratio_req);

/******************************************************************************
* Name:			MC_Cntms
* Description:	calculation of the number of sampling periods corresponding
*				to a given interval in ms
* Arguments:	interval duration in ms millisec
* Return Value:	corresponding number of sampling periods
******************************************************************************/
uint32_t MC_Cntms(uint16_t millisec);

/******************************************************************************
* Function Name: MC_SetOff
* Description  : current offsets measurement
* Arguments    : none
* Return Value : none
******************************************************************************/
void MC_SetOff(void);

/******************************************************************************
* Function Name: MC_IniPWM
* Description  : pwm initialization
* Arguments    : none
* Return Value : none
******************************************************************************/
void MC_IniPWM(void);

/******************************************************************************
* Function Name: MC_StartOut
* Description  : enable pwm outputs and insert a loop for bootstrap
* Arguments    : none
* Return Value : none
******************************************************************************/
void MC_StartOut(void);

/******************************************************************************
* Function Name: MC_StopOut
* Description  : disable pwm outputs
* Arguments    : none
* Return Value : none
******************************************************************************/
void MC_StopOut(void);

/******************************************************************************
* Function Name: MC_EnablePOE
* Description  : POE interrupt enable
* Arguments    : none
* Return Value : none
******************************************************************************/
void MC_EnablePOE(void);

/******************************************************************************
* Function Name: MC_StartPWM
* Description  : start pwm and main interrupt (but not the outputs)
* Arguments    : none
* Return Value : none
******************************************************************************/
void MC_StartPWM(void);

/******************************************************************************
* Function Name: MC_IniPar
* Description  : internal parameters initialization, using eeprom values
* Arguments    : none
* Return Value : none
******************************************************************************/
void MC_IniPar(void);

/******************************************************************************
* Function Name: MC_UpdPar
* Description  : internal parameters update, if eeprom values are changed
* Arguments    : none
* Return Value : none
******************************************************************************/
void MC_UpdPar(void);

/******************************************************************************
* Function Name: MC_ConFil
* Description  : management of measured values, to give them to the GUI
* Arguments    : none
* Return Value : none
******************************************************************************/
void MC_ConFil(void);

/******************************************************************************
* Function Name: MC_WorkMan
* Description  : Speed ramps (and special working modes) managements
* Arguments    : none
* Return Value : none
******************************************************************************/
void MC_WorkMan(void);

/******************************************************************************
* Function Name: MC_ConInt
* Description  : control interrupt
* Arguments    : none
* Return Value : none
* Note         : this function is called every sampling instant, and performs
*                the motor control
******************************************************************************/
void MC_ConInt(void);

/******************************************************************************
* Function Name: MC_FaultInt
* Description  : interrupt routine for POE0 (/fault from power stage)
* Arguments    : none
* Return Value : none
* Note         : this function stops the PWM output
******************************************************************************/
void MC_FaultInt(void);


#endif	// _MOTORCONTROL_H

