
/*******************************************************************************
*	DISCLAIMER
*
*	THIS SOFTWARE IS PROVIDED "AS IS" AND BFG ENGINEERING MAKES NO WARRANTIES
*	REGARDING THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING
*	BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
*	PURPOSE AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
*
*	WE (BFG ENGINEERING) DO NOT WARRANT THAT THIS SOFTWARE IS FREE FROM CLAIMS
*	BY A THIRD PARTY OF COPYRIGHT, PATENT, TRADEMARK, TRADE SECRET OR ANY OTHER
*	INTELLECTUAL PROPERTY INFRINGIMENT.
*
*	TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, BFG ENGINEERING
*	SHALL NOT BE LIABLE FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR
*	CONSEQUENTIAL DAMAGES FOR ANY REASON RELATED TO THIS SOFTWARE, EVEN IF BFG
*	ENGINEERING HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
*	PARTICULARLY, UNDER NO CIRCUMSTANCES ARE WE LIABLE FOR ANY OF THE FOLLOWING:
*	- THIRD PARTY CLAIMS AGAINST YOU FOR LOSSES OR DAMAGES;
*	- LOSS OF, OR DAMAGE TO YOUR RECORDS OR DATA; OR
*	- ECONOMIC CONSEQUENTIAL DAMAGES (INCLUDING LOST PROFITS OR SAVINGS) OR
*	  INCIDENTAL DAMAGES, EVEN IF WE ARE INFORMED OF THEIR POSSIBILITY.
*
*	BFG ENGINEERING DOES NOT WARRANT UNINTERRUPTED OR ERROR_FREE OPERATION OF
*	THIS SOFTWARE.
*	BFG ENGINEERING HAS NO OBLIGATION TO PROVIDE SERVICE, DEFECT CORRECTION OR
*	ANY MANTEINANCE OF THIS SOFTWARE; OR TO SUPPLY ANY UPDATES OR ENHANCEMENTS
*	EVEN IF SUCH ARE, OR LATER BECOME, AVAILABLE.
*
*	IF YOU DOWNLOAD OR USE THIS SOFTWARE YOU AGREE TO THESE TERMS, AND DO IT AT
*	YOUR OWN RISK.
*******************************************************************************/

/*******************************************************************************
*	File:		pws_E6125A.h
*	Project:	YRotateItRX111_3s_V1
*	Version:	1.0
*	Date:		01-07-2014 (DD-MM-YYYY)
*	Author:		Andrea Ghirardello
*	Contents:	Internal power stage (E6125A) related definitions
*******************************************************************************/

#ifndef	_PWS_E6125A_H
#define _PWS_E6125A_H

// current conversion
#define RSHUNT_OHM		( 0.1 ) 			// shunt value [Ohm]
#define RSGAIN			( 5.0 )				// circuit gain

// current sign: the current is considered positive is going out from the inverter;
// if in the hardware this produces a voltage lower than the offset then the following
// macro should be uncommented
//#define CUR_REV

// DC bus voltage conversion
#define RVBUSH_OHM		( 47000.0 )			// high side split resistor [Ohm]
#define RVBUSL_OHM		( 5000.0 )			// 10K||10K low side split resistor [Ohm]

// deadtimes to be inserted by MTU
#define DEADTIM_NS		( 1000 )			// deatimes [ns]

// minimum on time of low arms, to guarantee bootstrap operations
// this time should guarantee that the lower switch commutates effectively for enough time to
// charge the bootstrap capacitor
#define MIN_TON_LOW_NS	( 1500 )			// [ns] larger than deadtimes

// voltages
#define VB_NOM_VOL		( 24 )				// nominal bus voltage [V]

// inverter voltage errors compensation
// (should be applied if you use exact integration flux est. method,
// or to obtain better results in auto-tuning, especially with IGBT power stages))
#define NOCOMP								// remove comment to avoid ANY compensation in modulation (this is a MOSFET power stage)
//#define INLCOMP							// compensation of voltage errors due to inverter non-linearities (only if NOCOMP is commented)

// theoretical voltage error is:	E(I)=sign(I)*[(VB*DT/(2*SP))+VE(|I|)]
// measured voltage error is:		E(I)=sign(I)*[vb_mea_vol*(INL_DT_NS/1000000000)*pwm_fre_mea_Hz+INL_TAB_VOL(|I|)],
// where vb_mea_vol and pwm_fre_mea_Hz are the bus voltage and the pwm frequency used for the measurements,
// while INL_DT_NS and INL_TAB_VOL are the results of the measurements, the parameters used for the compensation.

// NOTE: due to the tuning method used, a multiply by sqrt(3)/2 is needed, both for the voltage tabled values and for the dead-times,
// and also the current scale has to be adapted in the same way

#define T_INL_DELTAIN	( 0.866 * 0.1 )		// (sqrt(3)/2) * 0.1A
#define T_INL_DIN_INT	((uint16_t)(KI * T_INL_DELTAIN))

#define T_INL_INDMAX	( 31 )
#define T_INL_INDMAX_U	( 1 )				// used max index (to save calculations)
#define INL_TABDIM		(T_INL_INDMAX + 1)
#define T_INL_INMAX_U	((uint16_t)(T_INL_DIN_INT * T_INL_INDMAX_U))

#define INL_DT_NS		( 0.866 * 0.0 )

#define INL_01			( 0.866 * 0.0 )		//  1
#define INL_02			( 0.866 * 0.0 )		//  2
#define INL_03			( 0.866 * 0.0 )		//  3
#define INL_04			( 0.866 * 0.0 )		//  4
#define INL_05			( 0.866 * 0.0 )		//  5
#define INL_06			( 0.866 * 0.0 )		//  6
#define INL_07			( 0.866 * 0.0 )		//  7
#define INL_08			( 0.866 * 0.0 )		//  8
#define INL_09			( 0.866 * 0.0 )		//  9
#define INL_10			( 0.866 * 0.0 )		// 10
#define INL_11			( 0.866 * 0.0 )		// 11
#define INL_12			( 0.866 * 0.0 )		// 12
#define INL_13			( 0.866 * 0.0 )		// 13
#define INL_14			( 0.866 * 0.0 )		// 14
#define INL_15			( 0.866 * 0.0 )		// 15
#define INL_16			( 0.866 * 0.0 )		// 16
#define INL_17			( 0.866 * 0.0 )		// 17
#define INL_18			( 0.866 * 0.0 )		// 18
#define INL_19			( 0.866 * 0.0 )		// 19
#define INL_20			( 0.866 * 0.0 )		// 20
#define INL_21			( 0.866 * 0.0 )		// 21
#define INL_22			( 0.866 * 0.0 )		// 22
#define INL_23			( 0.866 * 0.0 )		// 23
#define INL_24			( 0.866 * 0.0 )		// 24
#define INL_25			( 0.866 * 0.0 )		// 25
#define INL_26			( 0.866 * 0.0 )		// 26
#define INL_27			( 0.866 * 0.0 )		// 27
#define INL_28			( 0.866 * 0.0 )		// 28
#define INL_29			( 0.866 * 0.0 )		// 29
#define INL_30			( 0.866 * 0.0 )		// 30
#define INL_31			( 0.866 * 0.0 )		// 31


#endif	// _PWS_E6125A_H

